// This is (slightly modified) code from Thomas Nussbaumer's "Fast Pixel Access Macros and Line Drawing" tutorial found at http://tigcc.ticalc.org/tut/drawline.html
#define PIXOFFSET(x,y)  (((y)<<5)-((y)<<1)+((x)>>3))
#define PIXADDR(p,x,y)  (((unsigned char*)(p))+PIXOFFSET((x),(y)))
#define PIXMASK(x)      ((unsigned char)(0x80>>((x)&7)))
#define GETPIX(p,x,y)   (!(!(*PIXADDR(p,(x),(y))&PIXMASK(x))))
#define SETPIX(p,x,y)   (*PIXADDR(p,(x),(y))|= PIXMASK(x))
#define CLRPIX(p,x,y)   (*PIXADDR(p,(x),(y))&=~PIXMASK(x))
#define XORPIX(p,x,y)   (*PIXADDR(p,(x),(y))^= PIXMASK(x))
#define ABS(a)  (((a)<0) ? -(a): (a))
#define SGN(a)  (((a)<0) ? -1 : 1)
inline void drawline(unsigned char* plane,short x1,short y1,short x2,short y2,short mode) {
	short d,x,y,ax,ay,sx,sy,dx,dy;
	dx=x2-x1;
	ax=ABS(dx)<<1;
	sx=SGN(dx);
	dy=y2-y1;
	ay=ABS(dy)<<1;
	sy=SGN(dy);
	x=x1;
	y=y1;
	if (mode==A_NORMAL) {
		if (ax>ay) { // x dominant
			d=ay-(ax>>1);
			while (1) {
				SETPIX(plane,x,y);
				if (x==x2) return;
				if (d>=0) {
					y+=sy;
					d-=ax;
				}
				x+=sx;
				d+=ay;
			}
		}
		else { // y dominant
			d=ax-(ay>>1);
			while (1) {
				SETPIX(plane,x,y);
				if (y==y2) return;
				if (d>=0) {
					x+=sx;
					d-=ay;
				}
				y+=sy;
				d+=ax;
			}
		}
	}
	if (mode==A_REVERSE) {
		if (ax>ay) { // x dominant
			d=ay-(ax>>1);
			while (1) {
				CLRPIX(plane,x,y);
				if (x==x2) return;
				if (d>=0) {
					y+=sy;
					d-=ax;
				}
				x+=sx;
				d+=ay;
			}
		}
		else { // y dominant
			d=ax-(ay>>1);
			while (1) {
				SETPIX(plane,x,y);
				if (y==y2) return;
				if (d>=0) {
					x+=sx;
					d-=ay;
				}
				y+=sy;
				d+=ax;
			}
		}
	}
	if (mode==A_XOR) {
		if (ax>ay) { // x dominant
			d=ay-(ax>>1);
			while (1) {
				XORPIX(plane,x,y);
				if (x==x2) return;
				if (d>=0) {
					y+=sy;
					d-=ax;
				}
				x+=sx;
				d+=ay;
			}
		}
		else { // y dominant
			d=ax-(ay>>1);
			while (1) {
				SETPIX(plane,x,y);
				if (y==y2) return;
				if (d>=0) {
					x+=sx;
					d-=ay;
				}
				y+=sy;
				d+=ax;
			}
		}
	}
}
